/* Beholder RMON ethernet network monitor,Copyright (C) 1993 DNPAP group */
/* See file COPYING 'GNU General Public Licence' for copyright details   */
#ifdef SUNOS
#include "nwnit/nwnit.h"
#else
#ifdef ULTRIX
#include "nwpf/nwpf.h"
#endif
#endif
#include "nwicmp/nwicmp.h"
#include "nwudp/nwudp.h"
#include "nwtcp/nwtcp.h"
#include "nwsnmp/nwsnmp.h"
#include "nw.h"  



NW_DG_IFACE *nwDgIface[] =
{
#ifdef SUNOS
    &NitDgIface,
#else
#ifdef ULTRIX
    &PfDgIface,
#endif
#endif
    &IcmpDgIface,  
    &nwUdpIface,
    &SnmpDgIface
}; 


NW_CO_IFACE *nwCoIface[] =
{
    &nwTcpIface
};
   


static BOOLEAN      Parse(char *addr, char *head);
static NW_DG_IFACE *DgFindIface(char *addr);
static NW_CO_IFACE *CoFindIface(char *addr);
      

BOOLEAN NwInit(void)
{			 
BOOLEAN rc = TRUE;

    int i, count;
    
    count = sizeof(nwDgIface)/sizeof(NW_DG_IFACE *);
    
    for (i=0;i<count;i++)
        if (nwDgIface[i]->init() == FALSE)
			rc = FALSE;
    
    count = sizeof(nwCoIface)/sizeof(NW_CO_IFACE *);
    
    for (i=0;i<count;i++)
        if (nwCoIface[i]->init() == FALSE)
			rc = FALSE;

    return rc;
}

BOOLEAN NwExit(void)
{			
BOOLEAN rc = TRUE;

    int i, count;
    
    count = sizeof(nwDgIface)/sizeof(NW_DG_IFACE *);
    
    for (i=0;i<count;i++)
        if (nwDgIface[i]->exit() == FALSE)
			rc = FALSE;
    
    count = sizeof(nwCoIface)/sizeof(NW_CO_IFACE *);
    
    for (i=0;i<count;i++)
        if (nwCoIface[i]->exit() == FALSE)
			rc = FALSE;
        
    return rc;
}


NW_DG *NwDgAccept(char *addr, NW_DG_RECV_CALL recv, void *parm)
{
    NW_DG_IFACE *iface;

    iface = DgFindIface(addr);
    if (iface == 0)
        return 0; 
        
    if (iface->accept==0)
        return 0;
      
    return iface->accept(addr, recv, parm);
}


NW_DG *NwDgConnect(char *addr, NW_DG_RECV_CALL recv, void *parm)
{
    NW_DG_IFACE *iface;

    iface = DgFindIface(addr);
    if (iface == 0)
        return 0;
    
    if (iface->connect==0)
        return 0;
      
    return iface->connect(addr, recv, parm);
}



BOOLEAN NwDgSend(NW_DG *dg, BYTE *frame, int length) 
{
    if (dg->iface->send==0)
        return FALSE;    
        
    return dg->iface->send(dg, frame, length);
}    


BOOLEAN NwDgSendTo(NW_DG *dg, char *addr, BYTE *frame, int length) 
{
    NW_DG_IFACE *iface;
       
    if (dg==0)
        iface = DgFindIface(addr);
    else
        iface = dg->iface;
    
    if (iface==0)
        return FALSE;       
        
    if (iface->sendto==0)
        return FALSE;      
        
    return iface->sendto(dg, addr, frame, length);
}    


BOOLEAN NwDgClose(NW_DG *dg)
{
    if (dg == NULL)
        return TRUE;

    if (dg->iface->close==0)
        return FALSE;      
        
    return dg->iface->close(dg);
}    


NW_AC *NwCoAccept(char *addr, NW_CO_RECV_CALL recv, NW_CO_CLOSE_CALL close, NW_CO_OPEN_CALL open, void *parm)
{
    NW_CO_IFACE *iface;

    iface = CoFindIface(addr);
    if (iface == 0)
        return 0;       
        
    if (iface->accept==0)
        return 0;
      
    return iface->accept(addr, recv, close, open, parm);
}
    

NW_CO   *NwCoConnect (char *addr, NW_CO_RECV_CALL recv, NW_CO_CLOSE_CALL close, void *parm)
{
    NW_CO_IFACE *iface;

    iface = CoFindIface(addr);
    if (iface == 0)
        return 0;
        
    if (iface->connect==0)
        return 0;
      
    return iface->connect(addr, recv, close, parm);
}


BOOLEAN NwCoSend(NW_CO *co, BYTE *frame, int length)
{
    if (co->iface->send==0)
        return FALSE; 
             
    return co->iface->send(co, frame, length);
}    


BOOLEAN NwCoClose(NW_CO *co)
{
    if (co->iface->close==0)
        return FALSE; 
             
    return co->iface->close(co);
}    


static BOOLEAN Parse(char *addr, char *head)
{
    while (*addr!=':' && *addr!=0)
        *head++=*addr++;
    *head=0; 
       
    return TRUE;
} 
     
    
static NW_DG_IFACE *DgFindIface(char *addr)
{
    char descr[16]; 
    int  i, count;

    if (!Parse(addr, descr))
        return 0;   
    
    count = sizeof(nwDgIface)/sizeof(NW_DG_IFACE *);
    
    for (i=0;i<count;i++)
    {
        if (strcmp(descr, nwDgIface[i]->descr)==0)
            return nwDgIface[i]; 
    }
    
    return 0;
}

    
static NW_CO_IFACE *CoFindIface(char *addr)
{
    char descr[16]; 
    int  i, count;

    if (!Parse(addr, descr))
        return 0;   
    
    count = sizeof(nwCoIface)/sizeof(NW_CO_IFACE *);
    
    for (i=0;i<count;i++)
    {
        if (strcmp(descr, nwCoIface[i]->descr)==0)
            return nwCoIface[i]; 
    }
    
    return 0;
}


/*
 * Checksum routine for Internet Protocol family headers (C Version).
 *
 * Refer to "Computing the Internet Checksum" by R. Braden, D. Borman and
 * C. Partridge, Computer Communication Review, Vol. 19, No. 2, April 1989,
 * pp. 86-101, for additional details on computing this checksum.
 */

  
/* return checksum in low-order 16 bits */
int IpCheckSum(USHORT *ptr, INT nbytes)
{
	register long	 sum;           /* assumes long == 32 bits */
	u_short			 oddbyte;
	register u_short answer;        /* assumes u_short == 16 bits */

	/*
	 * Our algorithm is simple, using a 32-bit accumulator (sum),
	 * we add sequential 16-bit words to it, and at the end, fold back
	 * all the carry bits from the top 16 bits into the lower 16 bits.
	 */

	sum = 0;
	while (nbytes > 1)  {
		sum += *ptr++;
		nbytes -= 2;
	}

				/* mop up an odd byte, if necessary */
	if (nbytes == 1) {
		oddbyte = 0;        /* make sure top half is zero */
		*((u_char *) &oddbyte) = *(u_char *)ptr;   /* one byte only */
		sum += oddbyte;
	}

	/*
	 * Add back carry outs from top 16 bits to low 16 bits.
	 */

	sum  = (sum >> 16) + (sum & 0xffff);    /* add high-16 to low-16 */
	sum += (sum >> 16);                     /* add carry */
	answer = ~sum;          /* ones-complement, then truncate to 16 bits */
	return answer;
}
