/* Beholder RMON ethernet network monitor,Copyright (C) 1993 DNPAP group */
/* See file COPYING 'GNU General Public Licence' for copyright details   */
#ifndef _NW_H_
#define _NW_H_


#include <dnpap.h>


typedef struct timeval      TIMEVAL;
typedef struct sockaddr     SOCKADDR;
typedef struct sockaddr_in  SOCKADDR_IN;
typedef struct in_addr      IN_ADDR;
typedef struct strbuf       STRBUF;
typedef struct strioctl     STRIOCTL;
typedef struct ifreq        IFREQ;
typedef struct protoent     PROTOENT;
typedef struct hostent      HOSTENT;
typedef struct ip           IP;
typedef struct icmp         ICMP;


typedef struct _NW_DG NW_DG;
typedef struct _NW_CO NW_CO; 
typedef struct _NW_AC NW_AC; 
typedef struct _NW_DG_IFACE NW_DG_IFACE;
typedef struct _NW_CO_IFACE NW_CO_IFACE;


typedef void (*NW_DG_RECV_CALL)    (NW_DG *dg, char *addr, BYTE *frame, int length, void *parm);

typedef void (*NW_CO_RECV_CALL)    (NW_CO *co, BYTE *frame, int length, void *parm);
typedef void (*NW_CO_OPEN_CALL)    (NW_CO *co, char *addr, void *parm);
typedef void (*NW_CO_CLOSE_CALL)   (NW_CO *co, void *parm);

BOOLEAN  NwInit      (void); 
BOOLEAN  NwExit      (void);

NW_DG   *NwDgAccept  (char *addr, NW_DG_RECV_CALL recv, void *parm);
NW_DG   *NwDgConnect (char *addr, NW_DG_RECV_CALL recv, void *parm);
BOOLEAN  NwDgSend    (NW_DG *dg, BYTE *frame, int length);
BOOLEAN  NwDgSendTo  (NW_DG *dg, char *addr, BYTE *frame, int length);
BOOLEAN  NwDgClose   (NW_DG *dg);
 

NW_AC   *NwCoAccept  (char *addr, NW_CO_RECV_CALL recv, NW_CO_CLOSE_CALL close, NW_CO_OPEN_CALL open, void *parm);
NW_CO   *NwCoConnect (char *addr, NW_CO_RECV_CALL recv, NW_CO_CLOSE_CALL close, void *parm);
BOOLEAN  NwCoSend    (NW_CO *co, BYTE *frame, int length);
BOOLEAN  NwCoClose   (NW_CO *co);
BOOLEAN  NwCoRelease (NW_CO *co);



struct _NW_DG_IFACE
{
    char    *descr;
    BOOLEAN (*init)     (void); 
    BOOLEAN (*exit)     (void); 
    NW_DG   *(*accept)  (char *addr, NW_DG_RECV_CALL recv, void *parm); 
    NW_DG   *(*connect) (char *addr, NW_DG_RECV_CALL recv, void *parm);
    BOOLEAN (*send)     (NW_DG *dg, BYTE *frame, int length);  
    BOOLEAN (*sendto)   (NW_DG *dg, char *addr, BYTE *frame, int length);  
    BOOLEAN (*close)    (NW_DG *dg);
};


struct _NW_CO_IFACE
{
    char     *descr;
    BOOLEAN  (*init)    (void); 
    BOOLEAN  (*exit)    (void); 
    NW_AC    *(*accept) (char *addr, NW_CO_RECV_CALL recv, NW_CO_CLOSE_CALL close, NW_CO_OPEN_CALL open, void *parm);
    NW_CO    *(*connect)(char *addr, NW_CO_RECV_CALL recv, NW_CO_CLOSE_CALL close, void *parm);
    BOOLEAN  (*send)    (NW_CO *co, BYTE *frame, int length);
    BOOLEAN  (*close)   (NW_CO *co);
    BOOLEAN  (*release) (NW_AC *ac);
};


struct _NW_DG
{
    NW_DG_IFACE         *iface;   
    char                addr[32];
    NW_DG_RECV_CALL     recv;
    void                *parm;
};


struct _NW_CO
{            
    NW_CO_IFACE         *iface;
    char                addr[32];
    NW_CO_RECV_CALL     recv; 
    NW_CO_CLOSE_CALL    close;
    void                *parm;
};


struct _NW_AC
{            
    NW_CO_IFACE         *iface;
    char                addr[32];
    NW_CO_RECV_CALL     recv; 
    NW_CO_CLOSE_CALL    close;
    NW_CO_OPEN_CALL     open;
    void                *parm;  
};


int IpCheckSum(USHORT *ptr, INT nbytes);


#endif
